package com.fastest.pass.home.presentation.ui.components

import android.util.Log
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.gestures.detectTapGestures
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.BoxScope
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.ColumnScope
import androidx.compose.foundation.layout.ExperimentalLayoutApi
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.defaultMinSize
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.imeNestedScroll
import androidx.compose.foundation.layout.imePadding
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Switch
import androidx.compose.material3.SwitchDefaults
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.scale
import androidx.compose.ui.focus.FocusManager
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.ColorFilter
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.platform.LocalSoftwareKeyboardController
import androidx.compose.ui.platform.SoftwareKeyboardController
import androidx.compose.ui.res.colorResource
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.PasswordVisualTransformation
import androidx.compose.ui.text.input.VisualTransformation
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.fastest.pass.R
import com.fastest.pass.app.Utils
import com.fastest.pass.helpers.BasePreferenceHelper
import com.fastest.pass.home.data.model.NewItemFormField
import com.fastest.pass.home.domain.model.PasswordFormData
import javax.inject.Inject

@Composable
fun AddPasswordFormScreen(
    onPasswordsFormList: (ArrayList<NewItemFormField>, Boolean, Boolean) -> Unit,
    basePreferenceHelper: BasePreferenceHelper
) {
    val keyboardController = LocalSoftwareKeyboardController.current
    val focusManager = LocalFocusManager.current
    var passwords_title by remember { mutableStateOf("") }
    var passwords_url by remember { mutableStateOf("") }
    var passwords_username by remember { mutableStateOf("") }
    var passwords_password by remember { mutableStateOf("") }
    var passwords_notes by remember { mutableStateOf("") }

    Box(
        modifier = Modifier
            .fillMaxSize()
            .padding(horizontal = 30.dp)
            .padding(bottom = 20.dp)
            .background(Color.Transparent)
            .imePadding()
    ) {
        SaveButtonAPWFS(
            buttonText = R.string.save,
            onSaveButtonCLick = {
                val passwordsFormList = ArrayList<NewItemFormField>()
                val saltKey = basePreferenceHelper.getCustomSaltKey()
                val secretKey = basePreferenceHelper.getCustomSecretKey()

                val isTitleEmpty = passwords_title.isEmpty()
                val isPasswordEmpty = passwords_password.isEmpty()

                passwords_title = encryptIfNotEmpty(passwords_title, secretKey, saltKey)
                passwords_url = encryptIfNotEmpty(passwords_url, secretKey, saltKey)
                passwords_username = encryptIfNotEmpty(passwords_username, secretKey, saltKey)
                passwords_password = encryptIfNotEmpty(passwords_password, secretKey, saltKey)
                passwords_notes = encryptIfNotEmpty(passwords_notes, secretKey, saltKey)

                passwordsFormList.add(NewItemFormField("passwords_title", passwords_title))
                passwordsFormList.add(NewItemFormField("passwords_url", passwords_url))
                passwordsFormList.add(NewItemFormField("passwords_username", passwords_username))
                passwordsFormList.add(NewItemFormField("passwords_password", passwords_password))
                passwordsFormList.add(NewItemFormField("passwords_notes", passwords_notes))

                onPasswordsFormList.invoke(passwordsFormList, isTitleEmpty, isPasswordEmpty)
            }
        )

        Column(
            modifier = Modifier
                .fillMaxSize()
                .padding(horizontal = 0.dp)
                .padding(bottom = 80.dp)
                .background(Color.Transparent)
                .verticalScroll(rememberScrollState())
        ) {
            Spacer(modifier = Modifier.height(20.dp))
            TitleTextFieldAPS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                onTitleText = { passwords_title = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            UrlTextFieldAPS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                onUrlText = { passwords_url = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            UsernameTextFieldAPS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                onUsernameText = { passwords_username = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            PasswordTextFieldAPS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                onPasswordText = { passwords_password = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NotesTextFieldAPS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                onNotesText = { passwords_notes = it }
            )
            Spacer(modifier = Modifier.height(25.dp))
            OptionsText()
            Spacer(modifier = Modifier.height(25.dp))
            AutoFillOption()
//            Spacer(modifier = Modifier.height(25.dp))
//            SaveButtonAPFS(buttonText = R.string.save)
        }
    }

}

@Composable
fun ColumnScope.TitleTextFieldAPS(
    keyboardController: SoftwareKeyboardController?,
    focusManager: FocusManager,
    onTitleText: (String) -> Unit
) {
    var titleText by remember { mutableStateOf("") }

    TextField(
        value = titleText,
        onValueChange = {
            titleText = it
            onTitleText.invoke(titleText)
        },
        textStyle = MaterialTheme.typography.displayMedium,
        modifier = Modifier
            .align(Alignment.Start)
            .fillMaxWidth()
            .defaultMinSize(minHeight = 60.dp)
            .wrapContentHeight()
            .border(
                1.dp,
                color = colorResource(id = R.color.gray_border_textfield),
                shape = RoundedCornerShape(16.dp)
            )
            .background(color = colorResource(id = R.color.transparent)),
        shape = RoundedCornerShape(16.dp),
//        placeholder = {
//            Text(
//                text = stringResource(id = R.string.enter_email_address),
//                color = colorResource(id = R.color.gray_splash),
//                style = MaterialTheme.typography.displayMedium
//            )
//        },
        label = {
            Text(text = stringResource(R.string.title),
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
//        leadingIcon = {
//            Image(
//                painter = painterResource(id = R.drawable.profile_circle),
//                contentDescription = "Title Logo",
//                modifier = Modifier
//                    .size(24.dp, 24.dp)
//            )
//        },
        colors = TextFieldDefaults.colors(
            focusedLabelColor = colorResource(id = R.color.gray_splash),
            unfocusedContainerColor = colorResource(id = R.color.transparent),
            focusedContainerColor = colorResource(id = R.color.transparent),
            focusedIndicatorColor = colorResource(id = R.color.transparent),
            disabledIndicatorColor = colorResource(id = R.color.transparent),
            unfocusedIndicatorColor = colorResource(id = R.color.transparent),
            cursorColor = colorResource(id = R.color.gray_splash),
        ),
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Text,
            imeAction = ImeAction.Next
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                focusManager.clearFocus()
                keyboardController?.hide()
            }
        ),
    )
}

@Composable
fun ColumnScope.UrlTextFieldAPS(
    keyboardController: SoftwareKeyboardController?,
    focusManager: FocusManager,
    onUrlText: (String) -> Unit
) {
    var urlText by remember { mutableStateOf("") }

    TextField(
        value = urlText,
        onValueChange = {
            urlText = it
            onUrlText.invoke(urlText)
        },
        textStyle = MaterialTheme.typography.displayMedium,
        modifier = Modifier
            .align(Alignment.Start)
            .fillMaxWidth()
            .defaultMinSize(minHeight = 60.dp)
            .wrapContentHeight()
            .border(
                1.dp,
                color = colorResource(id = R.color.gray_border_textfield),
                shape = RoundedCornerShape(16.dp)
            )
            .background(color = colorResource(id = R.color.transparent)),
        shape = RoundedCornerShape(16.dp),
//        placeholder = {
//            Text(
//                text = stringResource(id = R.string.enter_email_address),
//                color = colorResource(id = R.color.gray_splash),
//                style = MaterialTheme.typography.displayMedium
//            )
//        },
        label = {
            Text(text = stringResource(R.string.url),
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
//        leadingIcon = {
//            Image(
//                painter = painterResource(id = R.drawable.profile_circle),
//                contentDescription = "Title Logo",
//                modifier = Modifier
//                    .size(24.dp, 24.dp)
//            )
//        },
        trailingIcon = {
            IconButton(
                onClick = {
                    urlText = ""
                },
                ) {
                if (urlText.isNotEmpty()) {
                    Image(
                        painter = painterResource(id = R.drawable.clear_button),
                        contentDescription = "clear button",
                        modifier = Modifier.size(24.dp)
                    )
                }
            }
        },
        colors = TextFieldDefaults.colors(
            focusedLabelColor = colorResource(id = R.color.gray_splash),
            unfocusedContainerColor = colorResource(id = R.color.transparent),
            focusedContainerColor = colorResource(id = R.color.transparent),
            focusedIndicatorColor = colorResource(id = R.color.transparent),
            disabledIndicatorColor = colorResource(id = R.color.transparent),
            unfocusedIndicatorColor = colorResource(id = R.color.transparent),
            cursorColor = colorResource(id = R.color.gray_splash),
        ),
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Uri,
            imeAction = ImeAction.Next
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                focusManager.clearFocus()
                keyboardController?.hide()
            }
        ),
    )
}

@Composable
fun ColumnScope.UsernameTextFieldAPS(
    keyboardController: SoftwareKeyboardController?,
    focusManager: FocusManager,
    onUsernameText: (String) -> Unit
) {
    var usernameText by remember { mutableStateOf("") }

    TextField(
        value = usernameText,
        onValueChange = {
            usernameText = it
            onUsernameText.invoke(usernameText)
        },
        textStyle = MaterialTheme.typography.displayMedium,
        modifier = Modifier
            .align(Alignment.Start)
            .fillMaxWidth()
            .defaultMinSize(minHeight = 60.dp)
            .wrapContentHeight()
            .border(
                1.dp,
                color = colorResource(id = R.color.gray_border_textfield),
                shape = RoundedCornerShape(16.dp)
            )
            .background(color = colorResource(id = R.color.transparent)),
        shape = RoundedCornerShape(16.dp),
//        placeholder = {
//            Text(
//                text = stringResource(id = R.string.enter_email_address),
//                color = colorResource(id = R.color.gray_splash),
//                style = MaterialTheme.typography.displayMedium
//            )
//        },
        label = {
            Text(text = stringResource(R.string.username),
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
//        leadingIcon = {
//            Image(
//                painter = painterResource(id = R.drawable.profile_circle),
//                contentDescription = "Title Logo",
//                modifier = Modifier
//                    .size(24.dp, 24.dp)
//            )
//        },
        colors = TextFieldDefaults.colors(
            focusedLabelColor = colorResource(id = R.color.gray_splash),
            unfocusedContainerColor = colorResource(id = R.color.transparent),
            focusedContainerColor = colorResource(id = R.color.transparent),
            focusedIndicatorColor = colorResource(id = R.color.transparent),
            disabledIndicatorColor = colorResource(id = R.color.transparent),
            unfocusedIndicatorColor = colorResource(id = R.color.transparent),
            cursorColor = colorResource(id = R.color.gray_splash),
        ),
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Text,
            imeAction = ImeAction.Next
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                focusManager.clearFocus()
                keyboardController?.hide()
            }
        ),
    )
}

@Composable
fun ColumnScope.PasswordTextFieldAPS(
    keyboardController: SoftwareKeyboardController?,
    focusManager: FocusManager,
    onPasswordText: (String) -> Unit
) {
    var passwordText by remember { mutableStateOf("") }
    var passwordVisible by remember { mutableStateOf(false) }

    TextField(
        value = passwordText,
        onValueChange = {
            passwordText = it
            onPasswordText.invoke(passwordText)
        },
        textStyle = MaterialTheme.typography.displayMedium,
        modifier = Modifier
            .align(Alignment.Start)
            .fillMaxWidth()
            .defaultMinSize(minHeight = 60.dp)
            .wrapContentHeight()
            .border(
                1.dp,
                color = colorResource(id = R.color.gray_border_textfield),
                shape = RoundedCornerShape(16.dp)
            )
            .background(color = colorResource(id = R.color.transparent)),
        shape = RoundedCornerShape(16.dp),
//        placeholder = {
//            Text(
//                text = stringResource(id = R.string.enter_password),
//                color = colorResource(id = R.color.gray_splash),
//                style = MaterialTheme.typography.displayMedium
//            )
//        },
        label = {
            Text(text = stringResource(R.string.password),
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
        colors = TextFieldDefaults.colors(
            focusedLabelColor = colorResource(id = R.color.gray_splash),
            unfocusedContainerColor = colorResource(id = R.color.transparent),
            focusedContainerColor = colorResource(id = R.color.transparent),
            focusedIndicatorColor = colorResource(id = R.color.transparent),
            disabledIndicatorColor = colorResource(id = R.color.transparent),
            unfocusedIndicatorColor = colorResource(id = R.color.transparent),
            cursorColor = colorResource(id = R.color.gray_splash),
        ),
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Password,
            imeAction = ImeAction.Next
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                focusManager.clearFocus()
                keyboardController?.hide()
            }
        ),
        visualTransformation =
        if (passwordVisible) VisualTransformation.Companion.None
        else PasswordVisualTransformation(),

        trailingIcon = {
            val description = if (passwordVisible) "Hide Password"
            else "Show Password"

            IconButton(onClick = {
                passwordVisible = !passwordVisible
            })
            {
                if (passwordVisible) {
                    Image(
                        painter = painterResource(id = R.drawable.eye_open),
                        contentDescription = description,
                        modifier = Modifier.size(24.dp)
                    )
                }
                else {
                    Image(
                        painter = painterResource(id = R.drawable.eye_slash3x),
                        contentDescription = description,
                        modifier = Modifier
                            .size(24.dp),
                        colorFilter = ColorFilter.tint(
                            colorResource(id = R.color.gray_splash)
                        )
                    )
                }
            }
        }
    )
}

@Composable
fun ColumnScope.NotesTextFieldAPS(
    keyboardController: SoftwareKeyboardController?,
    focusManager: FocusManager,
    onNotesText: (String) -> Unit
) {
    var notesText by remember { mutableStateOf("") }

    TextField(
        value = notesText,
        onValueChange = {
            notesText = it
            onNotesText.invoke(notesText)
        },
        textStyle = MaterialTheme.typography.displayMedium,
        modifier = Modifier
            .align(Alignment.Start)
            .fillMaxWidth()
            .defaultMinSize(minHeight = 60.dp)
            .wrapContentHeight()
            .border(
                1.dp,
                color = colorResource(id = R.color.gray_border_textfield),
                shape = RoundedCornerShape(16.dp)
            )
            .background(color = colorResource(id = R.color.transparent))
        ,
        shape = RoundedCornerShape(16.dp),
//        placeholder = {
//            Text(
//                text = stringResource(id = R.string.enter_email_address),
//                color = colorResource(id = R.color.gray_splash),
//                style = MaterialTheme.typography.displayMedium
//            )
//        },
        label = {
            Text(text = stringResource(R.string.notes),
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
//        leadingIcon = {
//            Image(
//                painter = painterResource(id = R.drawable.profile_circle),
//                contentDescription = "Title Logo",
//                modifier = Modifier
//                    .size(24.dp, 24.dp)
//            )
//        },
        colors = TextFieldDefaults.colors(
            focusedLabelColor = colorResource(id = R.color.gray_splash),
            unfocusedContainerColor = colorResource(id = R.color.transparent),
            focusedContainerColor = colorResource(id = R.color.transparent),
            focusedIndicatorColor = colorResource(id = R.color.transparent),
            disabledIndicatorColor = colorResource(id = R.color.transparent),
            unfocusedIndicatorColor = colorResource(id = R.color.transparent),
            cursorColor = colorResource(id = R.color.gray_splash),
        ),
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Text,
            imeAction = ImeAction.Default
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                focusManager.clearFocus()
                keyboardController?.hide()
            }
        ),
    )
}

@Composable
fun ColumnScope.OptionsText() {
    Text(
        text = stringResource(id = R.string.options),
        color = colorResource(id = R.color.gray_splash),
        style = MaterialTheme.typography.titleMedium,
        lineHeight = 20.sp,
        modifier = Modifier
            .align(Alignment.Start)
    )
}

@Composable
fun ColumnScope.AutoFillOption() {
    var isSwitch by remember { mutableStateOf(false) }

    Column {
        Row(
            modifier = Modifier
                .fillMaxWidth()
                .height(30.dp),
            verticalAlignment = Alignment.CenterVertically
        ) {
            Text(
                text = stringResource(id = R.string.autofill),
                color = colorResource(id = R.color.gray_splash),
                style = MaterialTheme.typography.titleMedium,
                lineHeight = 20.sp,
                modifier = Modifier
            )
            Spacer(modifier = Modifier.weight(1F))
            Switch(
                checked = isSwitch,
                onCheckedChange = {
                    isSwitch = it
                },
                modifier = Modifier.scale(0.8F),
                colors = SwitchDefaults.colors(
                    checkedThumbColor = colorResource(id = R.color.white),
                    checkedTrackColor = colorResource(id = R.color.sky_green),
                    uncheckedThumbColor = colorResource(id = R.color.white),
                    uncheckedTrackColor = colorResource(id = R.color.gray_text),
                    uncheckedBorderColor = colorResource(id = R.color.gray_text),
                ),
                thumbContent = {}
            )
        }
        Text(
            text = stringResource(id = R.string.automatically_fill_credentials),
            color = colorResource(id = R.color.gray_text),
            style = MaterialTheme.typography.displayMedium,
            lineHeight = 20.sp,
            modifier = Modifier
                .align(Alignment.Start)
                .fillMaxWidth(fraction = 0.8f)
        )
    }
}

@Composable
fun ColumnScope.SaveButtonAPFS(buttonText: Int) {
    Button(
        modifier = Modifier
            .background(colorResource(id = R.color.transparent))
            .fillMaxWidth()
            .height(60.dp)
            .clickable() { },
        onClick = {},
        shape = RoundedCornerShape(15.dp),
//            border = BorderStroke(25.dp, colorResource(id = R.color.black)),
        colors = ButtonDefaults.buttonColors(
            contentColor = colorResource(id = R.color.white),
            containerColor = colorResource(id = R.color.red_login_button),
        ),
    )
    {
        Text(
            text = stringResource(id = buttonText),
            style = MaterialTheme.typography.bodyMedium.copy(
                fontSize = 20.sp,
                color = colorResource(id = R.color.white)
            ),
            textAlign = TextAlign.Center
        )
    }
}

@Composable
fun BoxScope.SaveButtonAPWFS(
    buttonText: Int,
    onSaveButtonCLick: () -> Unit
    ) {
    Button(
        modifier = Modifier
            .background(colorResource(id = R.color.transparent))
            .fillMaxWidth()
            .height(60.dp)
            .align(Alignment.BottomCenter)
            .clickable() { },
        onClick = {
            onSaveButtonCLick.invoke()
        },
        shape = RoundedCornerShape(15.dp),
//            border = BorderStroke(25.dp, colorResource(id = R.color.black)),
        colors = ButtonDefaults.buttonColors(
            contentColor = colorResource(id = R.color.white),
            containerColor = colorResource(id = R.color.red_login_button),
        ),
    )
    {
        Text(
            text = stringResource(id = buttonText),
            style = MaterialTheme.typography.bodyMedium.copy(
                fontSize = 20.sp,
                color = colorResource(id = R.color.white)
            ),
            textAlign = TextAlign.Center
        )
    }
}

fun encryptIfNotEmpty(value: String, secretKey: String?, saltKey: String?): String {
    return if (value.isNotEmpty()) {
        Utils.encryptData(value, secretKey ?: "", saltKey ?: "")
    } else {
        value
    }
}